#include <sqrlMod.h>
extern "C"{

	extern void impulse_triggered_event_handler(uint32_t events_pins_low_to_high, uint32_t event_pins_high_to_low);
	static app_gpiote_user_id_t m_example_user_id;
	void gpio_interrupt_init(void){
		uint32_t errcode;

		nrf_gpio_cfg_sense_input(24, NRF_GPIO_PIN_NOPULL, NRF_GPIO_PIN_SENSE_HIGH);
		//nrf_gpio_cfg_sense_input(18, NRF_GPIO_PIN_NOPULL, NRF_GPIO_PIN_SENSE_LOW);

		APP_GPIOTE_INIT(1);

		uint32_t pri =  NVIC_GetPriority(GPIOTE_IRQn);

		//initialize gpio interrupt
		errcode = app_gpiote_user_register(	&m_example_user_id,					//unique user id for interrupt
											(0),								//low-to-high enable
											(1<<24),								//high-to-low enable
											impulse_triggered_event_handler);	//interrupt callback function
		APP_ERROR_CHECK(errcode);

		errcode = app_gpiote_user_enable(m_example_user_id);
		APP_ERROR_CHECK(errcode);

		//nrf_gpio_cfg_input(24,NRF_GPIO_PIN_PULLDOWN);
	}
}

sqrlMod::sqrlMod(u16 moduleId, Node* node, ConnectionManager* cm, const char* name, u16 storageSlot)
	: Module(moduleId, node, cm, name, storageSlot)
{
	uint32_t retval;

	//Register callbacks n' stuff
	Logger::getInstance().enableTag("SQRL");

	//Save configuration to base class variables
	//sizeof configuration must be a multiple of 4 bytes
	configurationPointer = &configuration;
	configurationLength = sizeof(sqrlModuleConfiguration);

	gunCount = 0;
	gunNum = 0;

	//Start module configuration loading
	LoadModuleConfiguration();


	gyro = new ITG3701(5,6);			//data, clk
	nrf_delay_us(5000);
	accel = new ADXL375Z(1,2,3,4); 	//mosi,miso,clk,cs
	nrf_delay_us(5000);

	gyro->init();
	accel->init(ADXL375Z_RATE_800);

	//configure debug gpio pin
	//nrf_gpio_cfg_output(24);
	//nrf_gpio_pin_set(24);

	//initialize the gpio trigger to wake up the nrf with accelerometer interrupt
	gpio_interrupt_init();


}


#define NODE_ID_A	72
#define NODE_ID_B	1984
#define NODE_ID_TABLET 45
void sqrlMod::activeSamplingState(void)
{
	logt("SQRL", "IMPACT DETECTED");

	int16_t gx,ax;
	gx = this->gyro->getGyroX();
	this->accel->getX(&ax);
	logt("SQRL", "%d,%d", (int16_t)gx, ax);
	/*
	int16_t gx;
	uint8_t dataX[6];
	for(;;)
	{

		//gyro->getFifoSample(dataX);
		//gx= dataX[0] + (dataX[1]<<8);
		gx = gyro->getGyroX();
		logt("DATA", "%d", gx);
		nrf_delay_us(10000);
	}
	*/

	accel->clearIntFlag();
}

bool sqrlMod::TerminalCommandHandler(string commandName, vector<string> commandArgs)
{
	//Must be called to allow the module to get and set the config
	Module::TerminalCommandHandler(commandName, commandArgs);

	//React on commands, return true if handled, false otherwise

	if(commandName == "SQRLMOD"){
	    //Get the id of the target node
	    nodeID targetNodeId = atoi(commandArgs[0].c_str());
	    logt("SQRL", "Trying to sqrl node %u", targetNodeId);

	    //Send ping packet to that node
	    connPacketModuleRequest packet;
	    packet.header.messageType = MESSAGE_TYPE_MODULE_TRIGGER_ACTION;
	    packet.header.sender = node->persistentConfig.nodeId;
	    packet.header.receiver = targetNodeId;

	    packet.moduleId = moduleId;
	    packet.data[0] = sqrlModuleTriggerActionMessages::TRIGGER_SQRL;
	    packet.data[1] = 123;

	    cm->SendMessageToReceiver(NULL, (u8*)&packet, SIZEOF_CONN_PACKET_MODULE_REQUEST+2, true);

	    return true;
	}
	if(commandName == "GET_X"){
		int16_t gx,ax;
		gx = this->gyro->getGyroX();
		this->accel->getX(&ax);
		logt("SQRL", "%d,%d", (int16_t)gx, ax);

		//if(cm->connections[0]->partnerId = NODE_ID_TABLET){

			nodeID targetNodeId = 45;
			//Sending Packet to tell mesh how many are to be sent
			syncPacketModuleRequest packet_beg;
			packet_beg.header.hasMoreParts = 0;
			packet_beg.header.messageType = MESSAGE_TYPE_MODULE_TRIGGER_ACTION;
			packet_beg.header.sender = node->persistentConfig.nodeId;
			packet_beg.header.receiver = targetNodeId;

			packet_beg.count = 1;
			packet_beg.data[0] = sqrlModuleTriggerActionMessages::TRIGGER_IMPACT_BEGIN;
			packet_beg.data[1] = gx&0xff;
			packet_beg.data[2] = gx>>8;
			packet_beg.data[3] = 0;
			packet_beg.data[4] = 0;
			packet_beg.data[5] = 0;
			packet_beg.data[6] = 0;
			packet_beg.data[7] = 0;
			packet_beg.data[8] = 0;
			packet_beg.data[9] = 0;
			packet_beg.data[10] = 0;
			packet_beg.data[11] = 0;
			packet_beg.data[12] = 0;

			cm->SyncMessage((u8*)&packet_beg, SIZEOF_CONN_PACKET_MODULE_REQUEST+12, false);
		//}

		return true;
	}
	if(commandName == "SQRLSYNC"){
		    //Get the id of the target node
		    nodeID targetNodeId = atoi(commandArgs[0].c_str());
		    logt("SQRL", "Trying to sqrl node %u", targetNodeId);

		    //Send ping packet to that node
		    connPacketModuleRequest packet;
		    packet.header.messageType = MESSAGE_TYPE_SYNC;
		    packet.header.sender = node->persistentConfig.nodeId;
		    packet.header.receiver = targetNodeId;

		    packet.data[0] = 28;

		    cm->SyncMessage((u8*)&packet, SIZEOF_CONN_PACKET_PAYLOAD_DATA_1+1, false);

		    return true;
		}

	if(commandName == "SQRLGUN"){
		//Get the id of the target node
		nodeID targetNodeId = atoi(commandArgs[0].c_str());
		int numPackets = atoi(commandArgs[1].c_str());
		logt("SQRL", "Sqrl is gunning down node %u with %d packets", targetNodeId, numPackets);

		//Sending Packet to tell mesh how many are to be sent
		connPacketModuleRequest packetInit;
		packetInit.header.messageType = MESSAGE_TYPE_MODULE_TRIGGER_ACTION;
		packetInit.header.sender = node->persistentConfig.nodeId;
		packetInit.header.receiver = targetNodeId;

        packetInit.moduleId = moduleId;
	    packetInit.data[0] = sqrlModuleTriggerActionMessages::TRIGGER_GUN_INIT;
	    packetInit.data[1] = numPackets;

	    cm->SendMessageToReceiver(NULL, (u8*)&packetInit, SIZEOF_CONN_PACKET_MODULE_REQUEST+2, true);

	    connPacketModuleRequest packet;
	    packet.header.messageType = MESSAGE_TYPE_MODULE_TRIGGER_ACTION;
	    packet.header.sender = node->persistentConfig.nodeId;
	    packet.header.receiver = targetNodeId;

        packet.moduleId = moduleId;
	    packet.data[0] = sqrlModuleTriggerActionMessages::TRIGGER_GUN;

	    //Send multiple packet to that node
		for(int x = 0; x <= numPackets-1;x++){
			packet.data[1] = x;
			cm->SendMessageToReceiver(NULL, (u8*)&packet, SIZEOF_CONN_PACKET_MODULE_REQUEST+2, true);
		}

		//Sending Packet to tell mesh how many are to be sent
		connPacketModuleRequest packetEnd;
		packetEnd.header.messageType = MESSAGE_TYPE_MODULE_TRIGGER_ACTION;
		packetEnd.header.sender = node->persistentConfig.nodeId;
		packetEnd.header.receiver = targetNodeId;

		packetEnd.moduleId = moduleId;
		packetEnd.data[0] = sqrlModuleTriggerActionMessages::TRIGGER_GUN_END;
		cm->SendMessageToReceiver(NULL, (u8*)&packetEnd, SIZEOF_CONN_PACKET_MODULE_REQUEST+1, true);

		return true;

	}

	return false;
}

void sqrlMod::ConnectionPacketReceivedEventHandler(connectionPacket* inPacket, Connection* connection, connPacketHeader* packetHeader, u16 dataLength)
{
    //Must call superclass for handling
    Module::ConnectionPacketReceivedEventHandler(inPacket, connection, packetHeader, dataLength);

    //Filter trigger_action messages
    if(packetHeader->messageType == MESSAGE_TYPE_MODULE_TRIGGER_ACTION){
        connPacketModuleRequest* packet = (connPacketModuleRequest*)packetHeader;

        //Check if our module is meant and we should trigger an action
        if(packet->moduleId == moduleId){
            //It's a ping message
        	if(packet->data[1] == sqrlModuleTriggerActionMessages::TRIGGER_SQRL){

                //Inform the user
                logt("SQRL", "sqrly data: %d", packet->data[1]);

                //Send PING_RESPONSE
                connPacketModuleRequest outPacket;
                outPacket.header.messageType = MESSAGE_TYPE_MODULE_ACTION_RESPONSE;
                outPacket.header.sender = node->persistentConfig.nodeId;
                outPacket.header.receiver = packetHeader->sender;
                outPacket.moduleId = moduleId;
                outPacket.data[0] = sqrlModuleActionResponseMessages::SQRL_RESPONSE;
                outPacket.data[1] = packet->data[0];
                outPacket.data[2] = 111;

                cm->SendMessageToReceiver(NULL, (u8*)&outPacket, SIZEOF_CONN_PACKET_MODULE_REQUEST + 3, true);

            }



        	if(packet->data[1] == sqrlModuleTriggerActionMessages::TRIGGER_GUN_INIT){
        		logt("SQRL", "Be prepared to get wasted.... with %d packets", packet->data[1]);
        		gunNum = packet->data[2];
        		gunCount =0;
        	}

        	if(packet->data[1] == sqrlModuleTriggerActionMessages::TRIGGER_GUN){
        		int16_t ax,ay,az,gx,gy,gz;
        		ax = packet->data[2]  + (packet->data[3]<<8);
        		ay = packet->data[4]  + (packet->data[5]<<8);
        		az = packet->data[6]  + (packet->data[7]<<8);
        		gx = packet->data[10]  + (packet->data[11]<<8);
        		u8 counter = packet->data[8];
        	    //Inform the user
        	    logt("SQRL", "Counter:%d sample: ax:%d ay:%d az:%d gy:%d", ax,ay,az,gx,counter);
        	    gunCount++;
        	 }

        	if(packet->data[1] == sqrlModuleTriggerActionMessages::TRIGGER_GUN_END){
        		u8 gotEmAll = 0;
        		if(gunCount == gunNum){
        			gotEmAll = 1;
        		}

        		//Send PING_RESPONSE
        		connPacketModuleRequest outPacket;
        		outPacket.header.messageType = MESSAGE_TYPE_MODULE_TRIGGER_ACTION;
        		outPacket.header.sender = node->persistentConfig.nodeId;
        		outPacket.header.receiver = packetHeader->sender;
        		outPacket.moduleId = moduleId;
        		outPacket.data[0] = sqrlModuleActionResponseMessages::GUN_RESPONSE;
        		outPacket.data[1] = gotEmAll;
        		cm->SendMessageToReceiver(NULL, (u8*)&outPacket, SIZEOF_CONN_PACKET_MODULE_REQUEST + 2, true);
        	}

        	if(packet->data[1] == sqrlModuleActionResponseMessages::GUN_RESPONSE){
        		//Inform the user
        	    logt("SQRL", "Did you get fully Wasted? %d", packet->data[1]);
        	}

        }
    }
}


